==> go-ld-prime-v0.0.1.tar.gz: go-ld-prime-v0.0.1/codec/raw/codec_test.go <==
package raw

import (
	"bytes"
	"fmt"
	"io"
	"testing"

	qt "github.com/frankban/quicktest"
	"gitlab.dms3.io/dms3/public/go-cid"
	ld "gitlab.dms3.io/ld/go-ld-prime"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"
	basicnode "gitlab.dms3.io/ld/go-ld-prime/node/basic"
)

var tests = []struct {
	name string
	data []byte
}{
	{"Empty", nil},
	{"Plaintext", []byte("hello there")},
	{"JSON", []byte(`{"foo": "bar"}`)},
	{"NullBytes", []byte("\x00\x00")},
}

func TestRoundtrip(t *testing.T) {
	t.Parallel()

	for _, test := range tests {
		t.Run(test.name, func(t *testing.T) {
			nb := basicnode.Prototype.Bytes.NewBuilder()
			r := bytes.NewBuffer(test.data)

			err := Decode(nb, r)
			qt.Assert(t, err, qt.IsNil)
			node := nb.Build()

			buf := new(bytes.Buffer)
			err = Encode(node, buf)
			qt.Assert(t, err, qt.IsNil)

			qt.Assert(t, buf.Bytes(), qt.DeepEquals, test.data)
		})
	}
}

func TestRoundtripCidlink(t *testing.T) {
	t.Parallel()

	lp := cidlink.LinkPrototype{Prefix: cid.Prefix{
		Version:  1,
		Codec:    rawMulticodec,
		MhType:   0x13,
		MhLength: 4,
	}}
	node := basicnode.NewBytes([]byte("hello there"))

	lsys := cidlink.DefaultLinkSystem()

	buf := bytes.Buffer{}
	lsys.StorageWriteOpener = func(lnkCtx ld.LinkContext) (io.Writer, ld.BlockWriteCommitter, error) {
		return &buf, func(lnk ld.Link) error { return nil }, nil
	}
	lsys.StorageReadOpener = func(lnkCtx ld.LinkContext, lnk ld.Link) (io.Reader, error) {
		return bytes.NewReader(buf.Bytes()), nil
	}
	lnk, err := lsys.Store(ld.LinkContext{}, lp, node)

	qt.Assert(t, err, qt.IsNil)

	newNode, err := lsys.Load(ld.LinkContext{}, lnk, basicnode.Prototype__Any{})
	qt.Assert(t, err, qt.IsNil)
	qt.Assert(t, newNode, qt.DeepEquals, node)
}

// mustOnlyUseRead only exposes Read, hiding Bytes.
type mustOnlyUseRead struct {
	buf *bytes.Buffer
}

func (r mustOnlyUseRead) Read(p []byte) (int, error) {
	return r.buf.Read(p)
}

// mustNotUseRead exposes Bytes and makes Read always error.
type mustNotUseRead struct {
	buf *bytes.Buffer
}

func (r mustNotUseRead) Read(p []byte) (int, error) {
	return 0, fmt.Errorf("must not call Read")
}

func (r mustNotUseRead) Bytes() []byte {
	return r.buf.Bytes()
}

func TestDecodeBuffer(t *testing.T) {
	t.Parallel()

	var err error
	buf := bytes.NewBuffer([]byte("hello there"))

	err = Decode(
		basicnode.Prototype.Bytes.NewBuilder(),
		mustOnlyUseRead{buf},
	)
	qt.Assert(t, err, qt.IsNil)

	err = Decode(
		basicnode.Prototype.Bytes.NewBuilder(),
		mustNotUseRead{buf},
	)
	qt.Assert(t, err, qt.IsNil)
}

==> go-ld-prime-v0.0.2.tar.gz: go-ld-prime-v0.0.2/codec/raw/codec_test.go <==
package raw

import (
	"bytes"
	"fmt"
	"io"
	"testing"

	qt "github.com/frankban/quicktest"
	"gitlab.dms3.io/dms3/go-cid"
	ld "gitlab.dms3.io/ld/go-ld-prime"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"
	basicnode "gitlab.dms3.io/ld/go-ld-prime/node/basic"
)

var tests = []struct {
	name string
	data []byte
}{
	{"Empty", nil},
	{"Plaintext", []byte("hello there")},
	{"JSON", []byte(`{"foo": "bar"}`)},
	{"NullBytes", []byte("\x00\x00")},
}

func TestRoundtrip(t *testing.T) {
	t.Parallel()

	for _, test := range tests {
		t.Run(test.name, func(t *testing.T) {
			nb := basicnode.Prototype.Bytes.NewBuilder()
			r := bytes.NewBuffer(test.data)

			err := Decode(nb, r)
			qt.Assert(t, err, qt.IsNil)
			node := nb.Build()

			buf := new(bytes.Buffer)
			err = Encode(node, buf)
			qt.Assert(t, err, qt.IsNil)

			qt.Assert(t, buf.Bytes(), qt.DeepEquals, test.data)
		})
	}
}

func TestRoundtripCidlink(t *testing.T) {
	t.Parallel()

	lp := cidlink.LinkPrototype{Prefix: cid.Prefix{
		Version:  1,
		Codec:    rawMulticodec,
		MhType:   0x13,
		MhLength: 4,
	}}
	node := basicnode.NewBytes([]byte("hello there"))

	lsys := cidlink.DefaultLinkSystem()

	buf := bytes.Buffer{}
	lsys.StorageWriteOpener = func(lnkCtx ld.LinkContext) (io.Writer, ld.BlockWriteCommitter, error) {
		return &buf, func(lnk ld.Link) error { return nil }, nil
	}
	lsys.StorageReadOpener = func(lnkCtx ld.LinkContext, lnk ld.Link) (io.Reader, error) {
		return bytes.NewReader(buf.Bytes()), nil
	}
	lnk, err := lsys.Store(ld.LinkContext{}, lp, node)

	qt.Assert(t, err, qt.IsNil)

	newNode, err := lsys.Load(ld.LinkContext{}, lnk, basicnode.Prototype__Any{})
	qt.Assert(t, err, qt.IsNil)
	qt.Assert(t, newNode, qt.DeepEquals, node)
}

// mustOnlyUseRead only exposes Read, hiding Bytes.
type mustOnlyUseRead struct {
	buf *bytes.Buffer
}

func (r mustOnlyUseRead) Read(p []byte) (int, error) {
	return r.buf.Read(p)
}

// mustNotUseRead exposes Bytes and makes Read always error.
type mustNotUseRead struct {
	buf *bytes.Buffer
}

func (r mustNotUseRead) Read(p []byte) (int, error) {
	return 0, fmt.Errorf("must not call Read")
}

func (r mustNotUseRead) Bytes() []byte {
	return r.buf.Bytes()
}

func TestDecodeBuffer(t *testing.T) {
	t.Parallel()

	var err error
	buf := bytes.NewBuffer([]byte("hello there"))

	err = Decode(
		basicnode.Prototype.Bytes.NewBuilder(),
		mustOnlyUseRead{buf},
	)
	qt.Assert(t, err, qt.IsNil)

	err = Decode(
		basicnode.Prototype.Bytes.NewBuilder(),
		mustNotUseRead{buf},
	)
	qt.Assert(t, err, qt.IsNil)
}
